﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Telerik.Windows.Controls;
using Telerik.Windows.Controls.GanttView;
using Telerik.Windows.Controls.Scheduling;

namespace GanttViewDependecyChain
{
    public class ViewModel : ViewModelBase
    {
        private ObservableCollection<CustomGanttTask> tasks;
        private CustomGanttTask selectedTask;
        private DateRange visibleTime;

        public ViewModel()
        {
            var date = DateTime.Now;
            this.tasks = GetTasks(date);
            this.visibleTime = new DateRange(date.AddDays(-1), date.AddDays(11));
            this.SelectedTask = this.Tasks[0].Children[2] as CustomGanttTask;
        }

        public CustomGanttTask SelectedTask
        {
            get
            {
                return this.selectedTask;
            }

            set
            {
                if (this.selectedTask != value)
                {
                    this.selectedTask = value;
                    this.OnPropertyChanged(() => this.SelectedTask);
                }
            }
        }

        public ObservableCollection<CustomGanttTask> Tasks
        {
            get
            {
                return tasks;
            }

            set
            {
                tasks = value;
                OnPropertyChanged(() => Tasks);
            }
        }

        public DateRange VisibleTime
        {
            get
            {
                return this.visibleTime;
            }

            set
            {
                if (this.visibleTime != value)
                {
                    this.visibleTime = value;
                    this.OnPropertyChanged(() => this.VisibleTime);
                }
            }
        }

        private ObservableCollection<CustomGanttTask> GetTasks(DateTime date)
        {
            var ganttAPI = new CustomGanttTask()
            {
                Start = date,
                End = date.AddDays(2),
                Title = "Design public API",
                Description = "Description: Design public API",
            };
            var ganttRendering = new CustomGanttTask()
            {
                Start = date.AddDays(2).AddHours(8),
                End = date.AddDays(4),
                Title = "Gantt Rendering",
                Description = "Description: Gantt Rendering",
            };
            var ganttDemos = new CustomGanttTask()
            {
                Start = date.AddDays(4.5),
                End = date.AddDays(7),
                Title = "Gantt Demos",
                Description = "Description: Gantt Demos",
            };
            var milestone = new CustomGanttTask()
            {
                Start = date.AddDays(7),
                End = date.AddDays(7).AddHours(1),
                Title = "Review",
                Description = "Description: Review",
                IsMilestone = true,
            };

            ganttRendering.Dependencies.Add(new Dependency() { FromTask = ganttAPI });

            ganttDemos.Dependencies.Add(new Dependency() { FromTask = ganttRendering });

            // The order in which the AddChainedDependecies is called is important. Make sure that it is called in the order from top to bottom in 
            // order for all the previous tasks to have their 'DependencyChain' to be up to date.
            this.AddChainedDependecies(ganttRendering, ganttAPI);
            this.AddChainedDependecies(ganttDemos, ganttRendering);

            var iterationTask = new CustomGanttTask()
            {
                Start = date,
                End = date.AddDays(7),
                Title = "Iteration 1",
                Children = { ganttAPI, ganttRendering, ganttDemos, milestone },
            };

            ObservableCollection<CustomGanttTask> tasks = new ObservableCollection<CustomGanttTask>() { iterationTask };

            return tasks;
        }

        private void AddChainedDependecies(CustomGanttTask task, CustomGanttTask fromTask)
        {
            task.DependencyChain.Add(fromTask);
            foreach (var inheritedDependecy in fromTask.DependencyChain)
            {
                task.DependencyChain.Add(inheritedDependecy);
            }
        }
    }
}
